@@ -1,15 +1,20 @@
 package Forks::Queue;
 use strict;
 use warnings;
-use base 'Exporter';
 use Scalar::Util 'looks_like_number';
 use Carp;
 use Time::HiRes;
+use Config;
 
-our $VERSION = '0.03';
-our $DEBUG = 0;
+our $VERSION = '0.04';
+our $DEBUG = $ENV{FQ_DEBUG} || 0;
 our $DEFAULT_IMPL;
 
+our $NOTIFY_OK = $ENV{FQ_NOTIFY} // do {
+    $Config::Config{sig_name} =~ /\bIO\b/;
+};
+our $SLEEP_INTERVAL = $NOTIFY_OK ? 10 : 1;
+
 sub new {
     my ($pkg,%opts) = @_;
     $opts{impl} //= $DEFAULT_IMPL; # undocumented, included for convenience
@@ -52,8 +57,6 @@ sub get {
     }
 }
 
-# TODO: to emulate Thread::Queue, dequeue(COUNT) should block until
-#       COUNT elements are available
 sub dequeue { goto &get; }
 
 sub dequeue_timed {
@@ -61,6 +64,8 @@ sub dequeue_timed {
     my $timeout = CORE::shift;
     _validate_input($timeout, 'timeout', 0, 1);
     local $self->{_expire} = Time::HiRes::time + $timeout;
+    local $SLEEP_INTERVAL = $Forks::Queue::SLEEP_INTERVAL;
+    $SLEEP_INTERVAL /= 2 while $SLEEP_INTERVAL > 0.25 * $timeout;
     return @_ ? $self->dequeue(@_) : $self->dequeue;
 }
 
@@ -69,6 +74,8 @@ sub get_timed {
     my $timeout = CORE::shift;
     _validate_input($timeout, 'timeout', 0, 1);
     local $self->{_expire} = Time::HiRes::time + $timeout;
+    local $SLEEP_INTERVAL = $Forks::Queue::SLEEP_INTERVAL;
+    $SLEEP_INTERVAL /= 2 while $SLEEP_INTERVAL > 0.25 * $timeout;
     return @_ ? $self->get(@_) : $self->get;
 }
 
@@ -77,6 +84,8 @@ sub shift_timed {
     my $timeout = CORE::shift;
     _validate_input($timeout, 'timeout', 0, 1);
     local $self->{_expire} = Time::HiRes::time + $timeout;
+    local $SLEEP_INTERVAL = $Forks::Queue::SLEEP_INTERVAL;
+    $SLEEP_INTERVAL /= 2 while $SLEEP_INTERVAL > 0.25 * $timeout;
     return @_ ? $self->shift(@_) : $self->shift;
 }
 
@@ -85,6 +94,8 @@ sub pop_timed {
     my $timeout = CORE::shift;
     _validate_input($timeout, 'timeout', 0, 1);
     local $self->{_expire} = Time::HiRes::time + $timeout;
+    local $SLEEP_INTERVAL = $Forks::Queue::SLEEP_INTERVAL;
+    $SLEEP_INTERVAL /= 2 while $SLEEP_INTERVAL > 0.25 * $timeout;
     return @_ ? $self->pop(@_) : $self->pop;
 }
 
@@ -165,7 +176,7 @@ Forks::Queue - queue that can be shared across processes
 
 =head1 VERSION
 
-0.03
+0.04
 
 =head1 SYNOPSIS
 
@@ -682,5 +693,6 @@ See http://dev.perl.org/licenses/ for more information.
 #
 #     priorities
 #     Directory implementation (see Queue::Dir)
-#     use SIGIO to wait for tasks/capacity, not just sleep
-#     what happens if you join a fifo queue as lifo?
+# _X_ use SIGIO to wait for tasks/capacity, not just sleep
+#         pros: better resolution, response than sleep 1
+#         cons: POSIX only, handled simultaneously by all queues in a process,
