@@ -8,10 +8,12 @@ use Time::HiRes;
 use base 'Forks::Queue';
 use 5.010;    #  using  // //=  operators
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 our $DEBUG;
 *DEBUG = \$Forks::Queue::DEBUG;
 
+$SIG{IO} = sub { } if $Forks::Queue::NOTIFY_OK;
+
 
 # prefer functional JSON calls because we still want to use JSON
 # during global destruction, and a JSON object might not be available
@@ -35,8 +37,13 @@ sub _SYNC (&$) {
     if ($self->{_locked}) {
         Carp::cluck "$$ acquiring lock but already have lock";
     }
+    local $! = 0;
     open my $lock, '>>', $self->{lock};
     my $z = flock $lock, 2;
+    while (!$z && $Forks::Queue::NOTIFY_OK && $!{EINTR}) {
+        # SIGIO can interrupt flock
+        $z = flock $lock, 2;
+    }
     $self->{_locked} = 1;
 
     $DEBUG and print STDERR "$$ Locked $self->{lock} z=$z\n";
@@ -117,7 +124,9 @@ sub new {
         $self->{_fh} = *$fh;
         seek $fh, 0, 0;
 
+        $self->{_locked}++;
         $self->_write_header;
+        $self->{_locked}--;
         if (tell($fh) < $self->{_header_size}) {
             print $fh "\0" x ($self->{_header_size} - tell($fh));
         }
@@ -168,6 +177,7 @@ sub DESTROY {
 
 sub _read_header {
     my ($self) = @_;
+    Carp::cluck "unsafe _read_header" unless $self->{_locked};
     seek $self->{_fh}, 0, 0;
     local $/ = EOL;
     chomp(my $h = readline($self->{_fh}));
@@ -195,6 +205,7 @@ sub _read_header {
 
 sub _write_header {
     my ($self) = @_;
+    Carp::cluck "unsafe _write_header" unless $self->{_locked};
     my $header = { index => $self->{_pos}, end => $self->{_end},
                    tell => $self->{_tell}, count => $self->{_count},
                    limit => $self->{limit},
@@ -215,6 +226,18 @@ sub _write_header {
     $DEBUG && print STDERR "$$ updated header $headerstr\n";
 }
 
+sub _notify {
+    return unless $Forks::Queue::NOTIFY_OK;
+
+    my $self = CORE::shift;
+    _SYNC { $self->_read_header } $self;
+    my @pids = grep { $_ != $$ } keys %{$self->{_pids}};
+    if (@pids) {
+        $DEBUG && print STDERR "$$ notify: @pids\n";
+        kill 'IO', @pids;
+    }
+}
+
 sub clear {
     my $self = CORE::shift;
     $self->_check_pid;
@@ -354,6 +377,7 @@ sub push {
         }
         $self->_write_header;
     } $self;
+    $self->_notify if $pushed;
 
     if ($failed_items) {
         if ($self->{on_limit} eq 'fail') {
@@ -380,7 +404,7 @@ sub _wait_for_item {
     do {
         _SYNC { $self->_read_header } $self;
         $ready = $self->{_avail} || $self->{_end} || $self->_expired;
-        sleep 1 if !$ready;
+        sleep($Forks::Queue::SLEEP_INTERVAL || 1) if !$ready;
     } while !$ready;
     return $self->{_avail};
 }
@@ -394,7 +418,7 @@ sub _wait_for_capacity {
         } else {
             _SYNC { $self->_read_header } $self;
             $ready = $self->{_avail} < $self->{limit} && !$self->{_end};
-            sleep 1 if !$ready;
+            sleep($Forks::Queue::SLEEP_INTERVAL || 1) if !$ready;
         }
     } while !$ready;
     return $self->{_avail} < $self->{limit};
@@ -432,8 +456,9 @@ sub _dequeue_back {
             $self->_write_header;
         } $self;
         last if @return || $self->{_end} || $self->_expired;
-        sleep 1;
+        sleep($Forks::Queue::SLEEP_INTERVAL || 1);
     }
+    $self->_notify if @return;
     if ($self->_expired && @return == 0) {
         return @_ ? $self->pop_nb(@_) : $self->pop_nb;
     }
@@ -462,6 +487,7 @@ sub _dequeue_front {
                     CORE::push @return, dejsonize($item);
                 };
                 if ($@) {
+                    $self->_write_header;
                     die "JSON was \"$item\", error was $@";
                 }
                 $self->{_pos}++;
@@ -476,8 +502,9 @@ sub _dequeue_front {
             $self->_write_header;
         } $self;
         last if @return || $self->{_end} || $self->_expired;
-        sleep 1;
+        sleep($Forks::Queue::SLEEP_INTERVAL || 1);
     }
+    $self->_notify if @return;
     if ($self->_expired && @return == 0) {
         return @_ ? $self->shift_nb(@_) : $self->shift_nb;
     }
@@ -500,20 +527,19 @@ sub shift {
             seek $self->{_fh}, $self->{_tell}, 0;
             while (@return < $count && $self->{_avail} > 0) {
                 my $item = readline($self->{_fh});
-                if (!defined($item)) {
-                    $self->_write_header;
-                    return;
+                if (defined($item)) {
+                    chomp($item);
+                    eval {
+                        CORE::push @return, dejsonize($item);
+                    };
+                    if ($@) {
+                        $self->_write_header;
+                        die "JSON was \"$item\", error was $@";
+                    }
+                    $self->{_pos}++;
+                    $self->{_tell} = tell $self->{_fh};
+                    $self->{_avail}--;
                 }
-		chomp($item);
-                eval {
-                    CORE::push @return, dejsonize($item);
-                };
-                if ($@) {
-                    die "JSON was \"$item\", error was $@";
-                }
-                $self->{_pos}++;
-                $self->{_tell} = tell $self->{_fh};
-                $self->{_avail}--;
             }
             if ($self->{_maintenance_freq} &&
                 $self->{_pos} >= $self->{_maintenance_freq}) {
@@ -521,9 +547,9 @@ sub shift {
                 $self->_maintain;
             }
             $self->_write_header;
-            return;
         } $self;
     }
+    $self->_notify if @return;
     if (!wantarray && @_ < 2) {
         return $return[0] // ();
     } else {
@@ -569,6 +595,7 @@ sub shift_nb {
         $self->_write_header;
         return;
     } $self;
+    $self->_notify if @return;
     if (!wantarray && @_ < 2) {
         return $return[0] // ();
     } else {
@@ -688,6 +715,7 @@ sub extract {
         truncate $self->{_fh}, tell $self->{_fh};
         $self->_write_header;
     } $self;
+    $self->_notify if @return;
     return @_ ? @return : $return[0] // ();
 }
 
@@ -773,6 +801,7 @@ sub insert {
             $inserted += $self->insert($pos+$inserted, @deferred_items);
         }
     }
+    $self->_notify if $inserted;
     return $inserted;
 }
 
@@ -786,13 +815,13 @@ sub pop {
         my $h;
         do {
             _SYNC { $self->_read_header } $self;
-        } while (!$self->{_avail} && !$self->{_end} && sleep 1);
+        } while (!$self->{_avail} && !$self->{_end} && 
+                 1 + sleep($Forks::Queue::SLEEP_INTERVAL || 1));
 
         return if $self->{_end} && !$self->{_avail};
 
         _SYNC {
             $self->_read_header;
-
             seek $self->{_fh}, $self->{_tell}, 0;
             if ($self->{_avail} <= $count) {
                 my @items = readline($self->{_fh});
@@ -800,25 +829,23 @@ sub pop {
                 @return = map dejsonize($_), @items;
                 truncate $self->{_fh}, $self->{_tell};
                 $self->{_count} -= @items;
-                $self->_write_header;
-                return;
+            } else {
+                my $pos = $self->{_pos};
+                while ($pos + $count < $self->{_count}) {
+                    scalar readline($self->{_fh});
+                    $pos++;
+                }
+                my $eof = tell $self->{_fh};
+                my @items = readline($self->{_fh});
+                truncate $self->{_fh}, $eof;
+                $self->{_count} -= @items;
+                chomp(@items);
+                @return = map dejsonize($_), @items;
             }
-
-            my $pos = $self->{_pos};
-            while ($pos + $count < $self->{_count}) {
-                scalar readline($self->{_fh});
-                $pos++;
-            }
-            my $eof = tell $self->{_fh};
-            my @items = readline($self->{_fh});
-            truncate $self->{_fh}, $eof;
-            $self->{_count} -= @items;
-	    chomp(@items);
-            @return = map dejsonize($_), @items;
             $self->_write_header;
-            return;
         } $self;
     }
+    $self->_notify if @return;
     if (!wantarray && @_ < 2) {
         return $return[0];
     } else {
@@ -864,6 +891,7 @@ sub pop_nb {
         $self->_write_header;
         return;
     } $self;
+    $self->_notify if @return;
     if (!wantarray && @_ < 2) {
         return $return[0];
     } else {
@@ -964,7 +992,7 @@ Forks::Queue::File - file-based implementation of Forks::Queue
 
 =head1 VERSION
 
-0.03
+0.04
 
 =head1 SYNOPSIS
 
