@@ -9,10 +9,12 @@ use Time::HiRes 'time';
 use base 'Forks::Queue';
 use 5.010;    #  using  // //=  operators
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 our $DEBUG;
 *DEBUG = \$Forks::Queue::DEBUG;
 
+$SIG{IO} = sub { } if $Forks::Queue::NOTIFY_OK;
+
 our $jsonizer = JSON->new->allow_nonref(1)->ascii(1);
 
 sub new {
@@ -45,7 +47,7 @@ sub new {
         my $dbh = DBI->connect("dbi:SQLite:dbname=" . $opts{db_file},
                                "", "");
         $self->{_dbh} = $opts{_dbh} = $dbh;
-        if (!$self->_init) {
+        if (!eval { $self->_init }) {
             carp "Forks::Queue::SQLite: db initialization failed";
             return;
         }
@@ -208,6 +210,7 @@ sub Forks::Queue::SQLite::MagicLimit::STORE {
     my $queue = $$tie;
     my $oldval  = $queue->{limit};
     $queue->{limit} = $val;
+
     my $dbh = $queue->_dbh;
     $dbh->begin_work;
     $queue->_status("limit",$val);
@@ -316,6 +319,7 @@ sub _push {
         @deferred_items = @items;
         $failed_items = @deferred_items;
     }
+    $self->_notify if $pushed;
 
     if ($failed_items) {
         if ($self->{on_limit} eq 'fail') {
@@ -326,7 +330,7 @@ sub _push {
                                    "Waiting for capacity\n";
             $self->_wait_for_capacity;
             $DEBUG && print STDERR "$$ got some capacity\n";
-            $pushed += $self->push(@deferred_items);
+            $pushed += $self->_push($tfactor,@deferred_items);
         }
     }
     return $pushed;
@@ -541,6 +545,7 @@ sub insert {
             $inserted += $self->insert($pos+$inserted,@deferred_items);
         }
     }
+    $self->_notify if $inserted;
     return $inserted;
 }
 
@@ -705,6 +710,21 @@ sub peek_back {
     return $self->_retrieve(+1,0,0,$index);
 }
 
+sub _notify {
+    return unless $Forks::Queue::NOTIFY_OK;
+
+    my $self = CORE::shift;
+    my $dbh = $self->_dbh;
+    my $sth = $dbh->prepare("SELECT pid FROM pids");
+    my $z = $sth->execute;
+    my $pt = $sth->fetchall_arrayref;
+    my @pids = grep { $_ != $$ } map { $_->[0] } @$pt;
+    if (@pids) {
+        $DEBUG && print STDERR "$$ notify: @pids\n";
+        kill 'IO', @pids;
+    }
+}
+
 my $id = 0;
 sub _impute_file {
     my $base = $0;
@@ -754,7 +774,7 @@ Forks::Queue::SQLite - SQLite-based implementation of Forks::Queue
 
 =head1 VERSION
 
-0.03
+0.04
 
 =head1 SYNOPSIS
 
