@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Carp;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 our $DEV_SHM = "/dev/shm";
 our $DEBUG;
 *DEBUG = \$Forks::Queue::DEBUG;
@@ -73,7 +73,9 @@ sub new {
         $self->{_fh} = *$fh;
         seek $fh, 0, 0;
 
+        $self->{_locked}++;
         $self->_write_header;
+        $self->{_locked}--;
         if (tell($fh) < $self->{_header_size}) {
             print $fh "\0" x ($self->{_header_size} - tell($fh));
         }
@@ -113,7 +115,7 @@ Forks::Queue::Shmem - Forks::Queue implementation using shared memory
 
 =head1 VERSION
 
-0.03
+0.04
 
 =head1 DESCRIPTION
 
